/*
-- =====================================================================
-- CUSTOPROD - Custo de Produtos
-- =====================================================================
*/
CREATE OR REPLACE PROCEDURE CUSTOPROD(
  P_CDFILIAL         IN PRODFILI.CDFILIAL%TYPE,
  P_CDCIA            IN COMPANHIA.CDCIA%TYPE,
  P_CDPRODUTO        IN PRODUTO.CDPRODUTO%TYPE,
  P_DATA             IN DATE,
  P_SGUNIDADE        IN PRODUTO.SGUNIDADE%TYPE,
  P_QTPROD           IN PRORECEITA.QTPRODREC%TYPE,
  VRPEFATOCORR      OUT PRODUTO.VRPEFATOCORR%TYPE,
  VRCUSTOULCLIQ_SAP OUT NUMBER,
  VRCUSTOULCLIQ_CAP OUT NUMBER,
  VRCUSTOULCBRU_SAP OUT NUMBER,
  VRCUSTOULCBRU_CAP OUT NUMBER,
  VRCUSTOMEDLIQ_SAP OUT NUMBER,
  VRCUSTOMEDLIQ_CAP OUT NUMBER,
  VRCUSTOMEDBRU_SAP OUT NUMBER,
  VRCUSTOMEDBRU_CAP OUT NUMBER,

  VRCUSTOULCLIQ_TSAP OUT NUMBER,
  VRCUSTOULCLIQ_TCAP OUT NUMBER,
  VRCUSTOULCBRU_TSAP OUT NUMBER,
  VRCUSTOULCBRU_TCAP OUT NUMBER,
  VRCUSTOMEDLIQ_TSAP OUT NUMBER,
  VRCUSTOMEDLIQ_TCAP OUT NUMBER,
  VRCUSTOMEDBRU_TSAP OUT NUMBER,
  VRCUSTOMEDBRU_TCAP OUT NUMBER)
AS
  V_CONTNULL         NUMBER;
  V_TESTENULL        NUMBER;
  V_VRCOMLIQ         NUMBER;
  V_VRCOMBRU         NUMBER;
  V_VRMEDLIQ         NUMBER;
  V_VRMEDBRU         NUMBER;

  V_VRPEFATOCORR     PRODUTO.VRPEFATOCORR%TYPE;
  V_VRFATOCONVES     CONVPRODUTO.VRFATOCONVES%TYPE;

  V_CUSTOULCLIQ_SAP  NUMBER;
  V_CUSTOULCLIQ_CAP  NUMBER;
  V_CUSTOULCBRU_SAP  NUMBER;
  V_CUSTOULCBRU_CAP  NUMBER;
  V_CUSTOMEDLIQ_SAP  NUMBER;
  V_CUSTOMEDLIQ_CAP  NUMBER;
  V_CUSTOMEDBRU_SAP  NUMBER;
  V_CUSTOMEDBRU_CAP  NUMBER;

  V_CUSTOULCLIQ_TSAP NUMBER;
  V_CUSTOULCLIQ_TCAP NUMBER;
  V_CUSTOULCBRU_TSAP NUMBER;
  V_CUSTOULCBRU_TCAP NUMBER;
  V_CUSTOMEDLIQ_TSAP NUMBER;
  V_CUSTOMEDLIQ_TCAP NUMBER;
  V_CUSTOMEDBRU_TSAP NUMBER;
  V_CUSTOMEDBRU_TCAP NUMBER;

  V_CONTPROREC       NUMBER;

BEGIN

   V_CUSTOULCLIQ_SAP := 0;
   V_CUSTOULCLIQ_CAP := 0;
   V_CUSTOULCBRU_SAP := 0;
   V_CUSTOULCBRU_CAP := 0;
   V_CUSTOMEDLIQ_SAP := 0;
   V_CUSTOMEDLIQ_CAP := 0;
   V_CUSTOMEDBRU_SAP := 0;
   V_CUSTOMEDBRU_CAP := 0;

   V_CUSTOULCLIQ_TSAP := 0;
   V_CUSTOULCLIQ_TCAP := 0;
   V_CUSTOULCBRU_TSAP := 0;
   V_CUSTOULCBRU_TCAP := 0;
   V_CUSTOMEDLIQ_TSAP := 0;
   V_CUSTOMEDLIQ_TCAP := 0;
   V_CUSTOMEDBRU_TSAP := 0;
   V_CUSTOMEDBRU_TCAP := 0;

	IF P_CDCIA IS NOT NULL  THEN
   	SELECT COUNT(*) INTO V_CONTPROREC
      FROM PRODUTOCIA
      WHERE (CDFILIAL  = P_CDFILIAL ) AND
            (CDCIA     = P_CDCIA    ) AND
            (CDPRODUTO = P_CDPRODUTO);
   ELSE
   	V_CONTPROREC := 0;
   END IF;

   IF V_CONTPROREC = 0 THEN

      SELECT COUNT(*)
      INTO V_TESTENULL
      FROM PRODUTO PR, CONVPRODUTO CP
      WHERE (PR.CDPRODUTO    = P_CDPRODUTO ) AND
            (CP.CDPRODUTO(+) = PR.CDPRODUTO) AND
            (CP.SGUNIDADE(+) = P_SGUNIDADE ) AND
            (PR.IDPRODCOMISS IS NULL       );

      IF V_TESTENULL > 0 THEN
         SELECT VRPEFATOCORR,VRFATOCONVES
         INTO V_VRPEFATOCORR,V_VRFATOCONVES
         FROM PRODUTO PR, CONVPRODUTO CP
         WHERE (PR.CDPRODUTO    = P_CDPRODUTO ) AND
               (CP.CDPRODUTO(+) = PR.CDPRODUTO) AND
               (CP.SGUNIDADE(+) = P_SGUNIDADE ) AND
               (PR.IDPRODCOMISS IS NULL       );
      ELSE
         V_VRPEFATOCORR := 0;
         V_VRFATOCONVES := 1;
      END IF;
      IF V_VRFATOCONVES IS NULL THEN
         V_VRFATOCONVES := 1;
      END IF;

      /*----------- ltima Compra e Mdio ---------------------------*/
      SELECT MAX(VRULTCLIQAP) AS VRCOMLIQ,
             MAX(VRULTCBRUAP) AS VRCOMBRU,
             MAX(VRMEDLIQAP ) AS VRMEDLIQ,
             MAX(VRMEDBRUAP ) AS VRMEDBRU
      INTO V_VRCOMLIQ,V_VRCOMBRU,V_VRMEDLIQ,V_VRMEDBRU
      FROM PRODUTOAP
      WHERE (CDFILIAL   = P_CDFILIAL)  AND
            (CDPRODUTO  = P_CDPRODUTO) AND
            (DTCUSTOAP  = (SELECT MAX(DTCUSTOAP)
                           FROM PRODUTOAP
                           WHERE (CDFILIAL   = P_CDFILIAL)  AND
                                 (CDPRODUTO  = P_CDPRODUTO) AND
                                 (DTCUSTOAP <= P_DATA     )));

      V_CUSTOULCLIQ_SAP := NVL(V_VRFATOCONVES * V_VRCOMLIQ,0);
      V_CUSTOULCBRU_SAP := NVL(V_VRFATOCONVES * V_VRCOMBRU,0);
      V_CUSTOMEDLIQ_SAP := NVL(V_VRFATOCONVES * V_VRMEDLIQ,0);
      V_CUSTOMEDBRU_SAP := NVL(V_VRFATOCONVES * V_VRMEDBRU,0);

      IF (V_CUSTOULCLIQ_SAP > 0) AND (V_CUSTOULCLIQ_SAP < 0.01) THEN
         V_CUSTOULCLIQ_SAP := 0.01;
      ELSE
         V_CUSTOULCLIQ_SAP := ROUND(V_CUSTOULCLIQ_SAP,3);
      END IF;
      IF (V_CUSTOULCBRU_SAP > 0) AND (V_CUSTOULCBRU_SAP < 0.01) THEN
         V_CUSTOULCBRU_SAP := 0.01;
      ELSE
         V_CUSTOULCBRU_SAP := ROUND(V_CUSTOULCBRU_SAP,3);
      END IF;
      IF (V_CUSTOMEDLIQ_SAP > 0) AND (V_CUSTOMEDLIQ_SAP < 0.01) THEN
         V_CUSTOMEDLIQ_SAP := 0.01;
      ELSE
         V_CUSTOMEDLIQ_SAP := ROUND(V_CUSTOMEDLIQ_SAP,3);
      END IF;
      IF (V_CUSTOMEDBRU_SAP > 0) AND (V_CUSTOMEDBRU_SAP < 0.01) THEN
         V_CUSTOMEDBRU_SAP := 0.01;
      ELSE
         V_CUSTOMEDBRU_SAP := ROUND(V_CUSTOMEDBRU_SAP,3);
      END IF;

      /* Se no existir na tabela ITLANCTOEST ou retornar zero. */
      IF V_CUSTOULCLIQ_SAP = 0 THEN
         SELECT MAX(VRULTCOMPR)
         INTO V_VRCOMLIQ
         FROM PRODFILI
         WHERE (CDPRODUTO = P_CDPRODUTO) AND
               (CDFILIAL  = P_CDFILIAL );

         V_CUSTOULCLIQ_SAP := NVL(V_VRFATOCONVES * V_VRCOMLIQ,0);
         V_CUSTOULCBRU_SAP := NVL(V_VRFATOCONVES * V_VRCOMLIQ,0);

         IF (V_CUSTOULCLIQ_SAP > 0) AND (V_CUSTOULCLIQ_SAP < 0.01) THEN
            V_CUSTOULCLIQ_SAP := 0.01;
         ELSE
            V_CUSTOULCLIQ_SAP := ROUND(V_CUSTOULCLIQ_SAP,3);
         END IF;
         IF (V_CUSTOULCBRU_SAP > 0) AND (V_CUSTOULCBRU_SAP < 0.01) THEN
            V_CUSTOULCBRU_SAP := 0.01;
         ELSE
            V_CUSTOULCBRU_SAP := ROUND(V_CUSTOULCBRU_SAP,3);
         END IF;
      END IF;

      /* Calcula o aproveitamento */
      IF V_VRPEFATOCORR = 100 THEN
      	V_CUSTOULCLIQ_CAP := 0;
         V_CUSTOULCBRU_CAP := 0;
         V_CUSTOMEDLIQ_CAP := 0;
         V_CUSTOMEDBRU_CAP := 0;
      ELSE
      	V_CUSTOULCLIQ_CAP := NVL(V_CUSTOULCLIQ_SAP / ((100 - V_VRPEFATOCORR)/100),0);
	      V_CUSTOULCBRU_CAP := NVL(V_CUSTOULCBRU_SAP / ((100 - V_VRPEFATOCORR)/100),0);
   	   V_CUSTOMEDLIQ_CAP := NVL(V_CUSTOMEDLIQ_SAP / ((100 - V_VRPEFATOCORR)/100),0);
      	V_CUSTOMEDBRU_CAP := NVL(V_CUSTOMEDBRU_SAP / ((100 - V_VRPEFATOCORR)/100),0);

         IF (V_CUSTOULCLIQ_CAP > 0) AND (V_CUSTOULCLIQ_CAP < 0.01) THEN
            V_CUSTOULCLIQ_CAP := 0.01;
         ELSE
            V_CUSTOULCLIQ_CAP := ROUND(V_CUSTOULCLIQ_CAP,3);
         END IF;
         IF (V_CUSTOULCBRU_CAP > 0) AND (V_CUSTOULCBRU_CAP < 0.01) THEN
            V_CUSTOULCBRU_CAP := 0.01;
         ELSE
            V_CUSTOULCBRU_CAP := ROUND(V_CUSTOULCBRU_CAP,3);
         END IF;
         IF (V_CUSTOMEDLIQ_CAP > 0) AND (V_CUSTOMEDLIQ_CAP < 0.01) THEN
            V_CUSTOMEDLIQ_CAP := 0.01;
         ELSE
            V_CUSTOMEDLIQ_CAP := ROUND(V_CUSTOMEDLIQ_CAP,3);
         END IF;
         IF (V_CUSTOMEDBRU_CAP > 0) AND (V_CUSTOMEDBRU_CAP < 0.01) THEN
            V_CUSTOMEDBRU_CAP := 0.01;
         ELSE
            V_CUSTOMEDBRU_CAP := ROUND(V_CUSTOMEDBRU_CAP,3);
         END IF;

      END IF;

      IF (V_CUSTOULCLIQ_SAP > 0) AND (V_CUSTOULCLIQ_SAP < 0.01) THEN
         V_CUSTOULCLIQ_SAP := 0.01;
      ELSE
         V_CUSTOULCLIQ_SAP := ROUND(V_CUSTOULCLIQ_SAP,3);
      END IF;
      IF (V_CUSTOULCBRU_SAP > 0) AND (V_CUSTOULCBRU_SAP < 0.01) THEN
         V_CUSTOULCBRU_SAP := 0.01;
      ELSE
         V_CUSTOULCBRU_SAP := ROUND(V_CUSTOULCBRU_SAP,3);
      END IF;
      IF (V_CUSTOMEDLIQ_SAP > 0) AND (V_CUSTOMEDLIQ_SAP < 0.01) THEN
         V_CUSTOMEDLIQ_SAP := 0.01;
      ELSE
         V_CUSTOMEDLIQ_SAP := ROUND(V_CUSTOMEDLIQ_SAP,3);
      END IF;
      IF (V_CUSTOMEDBRU_SAP > 0) AND (V_CUSTOMEDBRU_SAP < 0.01) THEN
         V_CUSTOMEDBRU_SAP := 0.01;
      ELSE
         V_CUSTOMEDBRU_SAP := ROUND(V_CUSTOMEDBRU_SAP,3);
      END IF;

      /* Joga a quantidade nos custos sem aproveitamento. */
      V_CUSTOULCLIQ_TSAP := ROUND(NVL(P_QTPROD * V_CUSTOULCLIQ_SAP,0),3);
      V_CUSTOULCBRU_TSAP := ROUND(NVL(P_QTPROD * V_CUSTOULCBRU_SAP,0),3);
      V_CUSTOMEDLIQ_TSAP := ROUND(NVL(P_QTPROD * V_CUSTOMEDLIQ_SAP,0),3);
      V_CUSTOMEDBRU_TSAP := ROUND(NVL(P_QTPROD * V_CUSTOMEDBRU_SAP,0),3);

      IF (V_CUSTOULCLIQ_CAP > 0) AND (V_CUSTOULCLIQ_CAP < 0.01) THEN
         V_CUSTOULCLIQ_CAP := 0.01;
      ELSE
         V_CUSTOULCLIQ_CAP := ROUND(V_CUSTOULCLIQ_CAP,3);
      END IF;
      IF (V_CUSTOULCBRU_CAP > 0) AND (V_CUSTOULCBRU_CAP < 0.01) THEN
         V_CUSTOULCBRU_CAP := 0.01;
      ELSE
         V_CUSTOULCBRU_CAP := ROUND(V_CUSTOULCBRU_CAP,3);
      END IF;
      IF (V_CUSTOMEDLIQ_CAP > 0) AND (V_CUSTOMEDLIQ_CAP < 0.01) THEN
         V_CUSTOMEDLIQ_CAP := 0.01;
      ELSE
         V_CUSTOMEDLIQ_CAP := ROUND(V_CUSTOMEDLIQ_CAP,3);
      END IF;
      IF (V_CUSTOMEDBRU_CAP > 0) AND (V_CUSTOMEDBRU_CAP < 0.01) THEN
         V_CUSTOMEDBRU_CAP := 0.01;
      ELSE
         V_CUSTOMEDBRU_CAP := ROUND(V_CUSTOMEDBRU_CAP,3);
      END IF;

      IF V_CUSTOMEDLIQ_SAP = 0 THEN
         V_CUSTOMEDLIQ_SAP := V_CUSTOULCLIQ_SAP;
         V_CUSTOMEDLIQ_CAP := V_CUSTOULCLIQ_CAP;
      END IF;
      IF V_CUSTOMEDBRU_SAP = 0 THEN
         V_CUSTOMEDBRU_SAP := V_CUSTOULCBRU_SAP;
         V_CUSTOMEDBRU_CAP := V_CUSTOULCBRU_CAP;
      END IF;

      /* Joga a quantidade nos custos com aproveitamento. */
      V_CUSTOULCLIQ_TCAP := ROUND(NVL(P_QTPROD * V_CUSTOULCLIQ_CAP,0),3);
      V_CUSTOULCBRU_TCAP := ROUND(NVL(P_QTPROD * V_CUSTOULCBRU_CAP,0),3);
      V_CUSTOMEDLIQ_TCAP := ROUND(NVL(P_QTPROD * V_CUSTOMEDLIQ_CAP,0),3);
      V_CUSTOMEDBRU_TCAP := ROUND(NVL(P_QTPROD * V_CUSTOMEDBRU_CAP,0),3);

	END IF;

   VRPEFATOCORR      := (100 - V_VRPEFATOCORR);
   VRCUSTOULCLIQ_SAP := V_CUSTOULCLIQ_SAP;
   VRCUSTOULCLIQ_CAP := V_CUSTOULCLIQ_CAP;
   VRCUSTOULCBRU_SAP := V_CUSTOULCBRU_SAP;
   VRCUSTOULCBRU_CAP := V_CUSTOULCBRU_CAP;
   VRCUSTOMEDLIQ_SAP := V_CUSTOMEDLIQ_SAP;
   VRCUSTOMEDLIQ_CAP := V_CUSTOMEDLIQ_CAP;
   VRCUSTOMEDBRU_SAP := V_CUSTOMEDBRU_SAP;
   VRCUSTOMEDBRU_CAP := V_CUSTOMEDBRU_CAP;

   VRCUSTOULCLIQ_TSAP := V_CUSTOULCLIQ_TSAP;
   VRCUSTOULCLIQ_TCAP := V_CUSTOULCLIQ_TCAP;
   VRCUSTOULCBRU_TSAP := V_CUSTOULCBRU_TSAP;
   VRCUSTOULCBRU_TCAP := V_CUSTOULCBRU_TCAP;
   VRCUSTOMEDLIQ_TSAP := V_CUSTOMEDLIQ_TSAP;
   VRCUSTOMEDLIQ_TCAP := V_CUSTOMEDLIQ_TCAP;
   VRCUSTOMEDBRU_TSAP := V_CUSTOMEDBRU_TSAP;
   VRCUSTOMEDBRU_TCAP := V_CUSTOMEDBRU_TCAP;

--DBMS_OUTPUT.PUT_LINE(P_CDPRODUTO||' - '||VRCUSTOMEDBRU_SAP);
END;
/
